/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <ostore/ostore.hh>
#include <ostore/coll.hh>
#include <ostore/relat.hh>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #4 - randomly choose "Query4RepeatCnt" document objects by 
// lookup on their title field.  An index can be used for the
// actual lookup.  
//
/////////////////////////////////////////////////////////////////


extern os_database* oo7db;

extern int	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #4
//
//////////////////////////////////////////////////////////////////


int  query4()
{
    // randomly select a document title and find the base assemblies
    // that use its associated composite part (and then repeat this
    // process the desired number of times)

    // set random seed so "hot" runs are truly hot
    srandom(1);

    int count = 0;
    char title[TitleSize];

    for (int i = 0; i < Query4RepeatCnt; i++) {

	// generate random document title

	int compPartId = (int) (random() % TotalCompParts) + 1;
        sprintf(title, "Composite Part %08d", compPartId);
        if (debugMode) {
            printf("    In Query4, document title = \"%s\"\n", title);
        }

        // now find the desired base assemblies

        os_Set<BaseAssembly*>& rslt =
	    (*BaseAssembly::extent)[:
	        componentsPriv[:
		    strcmp(title,documentation->title) == 0
		:]
	    :];

        // then process the relevant base assemblies

        foreach (BaseAssembly* base, rslt) {
	    base->DoNothing();
	    count++;
        }
    }
    return count;
}

