/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <ostore/ostore.hh>
#include <ostore/coll.hh>
#include <ostore/relat.hh>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #6 - find all assemblies (base or complex) B that "use" 
// (directly or transitively ) a composite part with a more recent 
// build date than B's build date.
//
/////////////////////////////////////////////////////////////////

extern os_database* oo7db;

extern int	debugMode;

//////////////////////////////////////////////////////////////////
//
// Recursive Workhorse Routine for Query #6
//
//////////////////////////////////////////////////////////////////


int checkMaxCompDate(Assembly* assm, int& count)
{
    // check to see if this assembly uses a more recently dated composite
    // part, and process it if so - handling complex and base assemblies
    // differently since the former have subassemblies while the latter
    // directly use composite parts

    int maxSubDate = -1;

    if (assm->myType() == Complex) {

        // complex assembly case - check its subassemblies recursively

        if (debugMode) {
            printf("In Query6:  complex assembly %d, buildDate = %d\n",
                    assm->id, assm->buildDate);
        }

	ComplexAssembly* compAssm = (ComplexAssembly*) assm;
	foreach (Assembly* subAssm, compAssm->subAssemblies) {
	    int rsltDate = checkMaxCompDate(subAssm, count);
	    if (rsltDate > maxSubDate) {
	        maxSubDate = rsltDate;
	    }
	}

    } else {

        // base assembly case - check its composite part build dates

        if (debugMode) {
            printf("In Query6:  base assembly %d, buildDate = %d\n",
                    assm->id, assm->buildDate);
            printf("    composite part dates = { ");
        }

	BaseAssembly* baseAssm = (BaseAssembly*) assm;
	foreach (CompositePart* comp, baseAssm->componentsPriv) {
            if (debugMode) { printf("%d ",comp->buildDate); }
	    if (comp->buildDate > maxSubDate) {
	        maxSubDate = comp->buildDate;
	    }
        }
        if (debugMode) { printf("}\n"); }
    }

    // see if the maximum build date of composite parts used by this assembly
    // is greater than the assembly's build date, and process the assembly if
    // this is found to be the case;  also, return the maximum build date for
    // this subassembly

    if (maxSubDate > assm->buildDate) {
        assm->DoNothing();
	count++;
    }
    return(maxSubDate);
}


//////////////////////////////////////////////////////////////////
//
// Top-Level Routine to do Query #6
//
//////////////////////////////////////////////////////////////////


int query6()
{
    // scan modules, visiting all of the assemblies in each one, checking
    // to see if they use (either directly or indirectly) a more recently
    // dated composite part;  the visits are done by recursively traversing
    // the assemblies, checking them on the way back up (to avoid lots of
    // repeated work)

    int count = 0;
    foreach (Module* mod, *Module::extent) {
	int rslt = checkMaxCompDate(mod->designRoot, count);
    }
    return count;
}

